C
C Version 20-03-2008
C         Wien2k
C
C Author  Maurits W. Haverkort
C         Written at Max Planck Institute Stuttgart
C
C         (C) 2008
C
C Spaghetti Primavera is a program that creates a band-structure plot (eps)
C given a list of k-points and a list of eigen-energies and eigen-functions
C or characters.
C
C Sperate functions can specify the line thickness and color of the bands
C as a function of the character.
C
C The Wien2k version expects a klist file and a qtl file
C
      program SpaghettiPrimavera
      implicit none
C
      character qtlName*100,bandName*100
      integer MinorTicks,nkpoints
      real*8 Xsize,Ysize,Emin,Emax,Efermi,MajorTicks,TextSize
C
      call ReadDefinitions(qtlName,bandName,Xsize,Ysize,Emin,Emax,
     &                     Efermi,MajorTicks,MinorTicks,TextSize)
      call WritePSHead(Xsize,Ysize)
      call WritePSKpoints(bandName,nkpoints, Xsize,Ysize,
     &                    TextSize)
      call WritePSAxesA(Xsize,Ysize,Emin,Emax,MajorTicks,
     &                  MinorTicks,TextSize)
      call WritePSBands(qtlName,nkpoints,Xsize,Ysize,
     &                  Efermi,TextSize,Emin,Emax)
      call WritePSAxesB(Xsize,Ysize,Emin,Emax,MajorTicks,
     &                  MinorTicks,TextSize)
      call WritePSTail()
C
100   FORMAT(A)
      end
C
      subroutine WritePSHead(Xsize,Ysize)
C
C writes the header of the postscript to standard output
C
      implicit none
      real*8 Xsize,Ysize
      write(6,100)"%!PS-Adobe-2.0 EPSF-2.0"
      write(6,101)"%%BoundingBox: 0 0",int(Xsize+100),int(Ysize+100)
      write(6,102)"%%HiResBoundingBox: 0.000000 0.000000",
     &Xsize+100,Ysize+100
      write(6,100)"%%Creator: Spaghetti Primavera
     & by Maurits W. Haverkort"
      write(6,100)"%%EndComments"
      write(6,100)"%%BeginProlog"
      write(6,100)"save"
      write(6,100)"countdictstack"
      write(6,100)"mark"
      write(6,100)"newpath"
      write(6,100)"/setpagedevice {pop} def"
      write(6,100)"/L{ newpath setlinewidth setrgbcolor moveto lineto
     & lineto 0 setlinecap 1 setlinejoin stroke } def"
      write(6,100)"/L1{ newpath setlinewidth setrgbcolor moveto
     & lineto 0 setlinecap 1 setlinejoin stroke } def"
      write(6,100)"%%EndProlog"
      write(6,100)"%%Page 1 1"
100   FORMAT(A)
101   FORMAT(A,2I5)
102   FORMAT(A,2F12.6)
      end
C
      subroutine WritePSTail()
C
C Writes the tail of the postscript to standard output
C
      implicit none
      write(6,100)"%%Trailer"
      write(6,100)"cleartomark"
      write(6,100)"countdictstack"
      write(6,100)"exch sub { end } repeat"
      write(6,100)"restore"
      write(6,100)"%%EOF"
100   FORMAT(A)
      end
C
      subroutine WritePSAxesA(Xsize,Ysize,Emin,Emax,MajorTicks,
     &                       MinorTicks,TextSize)
C
C Writes the part of the axes system to the standard output
C that should be behind the graph 
C
      implicit none
      real*8 Xsize,Ysize,Emin,Emax,MajorTicks,TextSize
      integer MinorTicks
C
      real*8 PtEfermi,PtYunit,PtTick
C A line at the fermi energy
      PtEfermi=50-(Ysize*Emin)/(Emax-Emin)
      PtYunit=(Ysize)/(Emax-Emin)
      if ((Emin.LT.0).AND.(Emax.GE.0)) then
        write(6,100)"newpath"
        write(6,100)"1 setlinewidth"
        write(6,100)"0.5 0.5 0.5 setrgbcolor"
        write(6,103)"50 ",PtEfermi," moveto"
        write(6,102)50+Xsize,PtEfermi," lineto"
        write(6,100)"stroke"
      endif
C
100   FORMAT(A)
101   FORMAT(F12.6,A)
102   FORMAT(2F12.6,A)
103   FORMAT(A,F12.6,A)
104   FORMAT(A,F5.1,A)
      end
C
      subroutine WritePSAxesB(Xsize,Ysize,Emin,Emax,MajorTicks,
     &                       MinorTicks,TextSize)
C
C Writes the part of the axes system that should be on top of the graph
C
      implicit none
      real*8 Xsize,Ysize,Emin,Emax,MajorTicks,TextSize
      integer MinorTicks
C
      real*8 PtEfermi,PtYunit,PtTick
C some positions in pts
      PtEfermi=50-(Ysize*Emin)/(Emax-Emin)
      PtYunit=(Ysize)/(Emax-Emin)
C write the ticks
      if((Emin+MajorTicks).LT.Emax) then
        write(6,100)"newpath"
        write(6,100)"1 setlinewidth"
        write(6,100)"0 0 0 setrgbcolor"
        PtTick=PtEfermi
        do while (PtTick.LT.(Ysize+50))
          write(6,103)"50 ",PtTick," moveto"
          write(6,102)50+Xsize/100.0,PtTick," lineto"
          write(6,102)50+Xsize,PtTick," moveto"
          write(6,102)50+Xsize-Xsize/100.0,PtTick," lineto"
          PtTick=PtTick+MajorTicks*PtYunit
        enddo
        PtTick=PtEfermi
        PtTick=PtTick-MajorTicks*PtYunit
        do while (PtTick.GE.(50))
          write(6,103)"50 ",PtTick," moveto"
          write(6,102)50+Xsize/100.0,PtTick," lineto"
          write(6,102)50+Xsize,PtTick," moveto"
          write(6,102)50+Xsize-Xsize/100.0,PtTick," lineto"
          PtTick=PtTick-MajorTicks*PtYunit
        enddo
        write(6,100)"stroke"
        write(6,100)"newpath"
        write(6,100)"0.75 setlinewidth"
        write(6,100)"0 0 0 setrgbcolor"
        PtTick=PtEfermi
        do while (PtTick.LT.(Ysize+50))
          write(6,103)"50 ",PtTick," moveto"
          write(6,102)50+Xsize/150.0,PtTick," lineto"
          write(6,102)50+Xsize,PtTick," moveto"
          write(6,102)50+Xsize-Xsize/150.0,PtTick," lineto"
          PtTick=PtTick+MajorTicks*PtYunit/(1.0+MinorTicks)
        enddo
        PtTick=PtEfermi
        PtTick=PtTick-MajorTicks*PtYunit/(1.0+MinorTicks)
        do while (PtTick.GE.(50))
          write(6,103)"50 ",PtTick," moveto"
          write(6,102)50+Xsize/150.0,PtTick," lineto"
          write(6,102)50+Xsize,PtTick," moveto"
          write(6,102)50+Xsize-Xsize/150.0,PtTick," lineto"
          PtTick=PtTick-MajorTicks*PtYunit/(1.0+MinorTicks)
        enddo
        write(6,100)"stroke"
      endif
C write the numbers next to the ticks
      write(6,100)"/Times-Roman findfont"
      write(6,101)TextSize," scalefont"
      write(6,100)"setfont"
      write(6,100)"0 0 0 setrgbcolor"
      PtTick=PtEfermi
      do while (PtTick.LT.(Ysize+50))
        write(6,102)50-2*TextSize,PtTick-TextSize/4.0," moveto"
        write(6,104)"(",(PtTick-PtEfermi)/PtYunit,") show"
        PtTick=PtTick+MajorTicks*PtYunit
      enddo
      PtTick=PtEfermi
      PtTick=PtTick-MajorTicks*PtYunit
      do while (PtTick.GE.(50))
        write(6,102)50-2*TextSize,PtTick-TextSize/4.0," moveto"
        write(6,104)"(",(PtTick-PtEfermi)/PtYunit,") show"
        PtTick=PtTick-MajorTicks*PtYunit
      enddo      
C write a frame around the graph
      write(6,100)"newpath"
      write(6,100)"1 setlinewidth"
      write(6,100)"0 setlinejoin"
      write(6,100)"0 0 0 setrgbcolor"
      write(6,100)"50 50 moveto"
      write(6,101)Xsize+50," 50 lineto"
      write(6,102)Xsize+50,Ysize+50," lineto"
      write(6,103)"50",Ysize+50," lineto"
      write(6,100)"closepath"
      write(6,100)"stroke"
C
100   FORMAT(A)
101   FORMAT(F12.6,A)
102   FORMAT(2F12.6,A)
103   FORMAT(A,F12.6,A)
104   FORMAT(A,F5.1,A)
      end
C
      subroutine WritePSKpoints(bandName,nkpoints,Xsize,Ysize,
     &                          TextSize)
C
C reads the file bandName and 
C makes vertical lines for each k-point that has a label in the 
C file band-name. These lines are labeled
C returns the number of points that should be between each k-point
C and the number of k-points
C
      implicit none
      character bandName*100
      real*8 Xsize,Ysize,TextSize
      integer nkpoints
C
      integer OK,nlabels,labelpoint(100),ilabel
      character RDWD*(128),label(100)*(2)
C
      open(unit=1,file=bandName,status="old",iostat=OK)
      if(OK.NE.0) then
        write(6,100)"/Times-Roman findfont"
        write(6,101)TextSize," scalefont"
        write(6,100)"setfont"
        write(6,100)"newpath"
        write(6,100)"1 0 0 setrgbcolor"
        write(6,103)"50 ",Ysize/2+50," moveto"
        write(6,100)"(Could not open file",bandName,") show"
        call WritePSTail()
        stop
      endif
C read file "bandName" and filter the number of k-points, 
C the labels and their position
      nkpoints=0
      nlabels=0
      read(unit=1,fmt=100,iostat=OK,err=200,end=201) RDWD
      do while (RDWD.NE."END")
        nkpoints=nkpoints+1
        if(RDWD(1:1).NE." ") then
          nlabels=nlabels+1
          label(nlabels)=RDWD(1:2)
          labelpoint(nlabels)=nkpoints
        endif
        read(unit=1,fmt=100,iostat=OK,err=200,end=201) RDWD
      enddo
C plot lines for specific k-points
      ilabel=1
      do while (ilabel.LT.nlabels-1)
        ilabel=ilabel+1
        write(6,100)"newpath"
        write(6,100)"1 setlinewidth"
        write(6,100)"0.5 0.5 0.5 setrgbcolor"
        write(6,101)50+(labelpoint(ilabel)-1.0)*Xsize/(nkpoints-1.0),
     &              " 50 moveto"
        write(6,102)50+(labelpoint(ilabel)-1.0)*Xsize/(nkpoints-1.0),
     &              50+Ysize," lineto"
        write(6,100)"stroke"
      enddo
C write the labels for specific k-points
      ilabel=0
      do while (ilabel.LT.nlabels)
        ilabel=ilabel+1
        if (label(ilabel)(2:2).EQ." ") then
          write(6,100)"/Times-Roman findfont"
        else
          write(6,100)"/Symbol findfont"
        endif
        write(6,101)TextSize," scalefont"
        write(6,100)"setfont"
        write(6,100)"0 0 0 setrgbcolor"
        write(6,102)50+(labelpoint(ilabel)-1.0)*Xsize/(nkpoints-1.0)-
     &              TextSize/2.0,50-TextSize," moveto"
        write(6,100)"(",label(ilabel)(1:1),") show"
      enddo
      close(1)
      return
C Goto 200 if the file "bandName" had an error upon reading
200   write(6,100)"/Times-Roman findfont"
      write(6,101)TextSize," scalefont"
      write(6,100)"setfont"
      write(6,100)"newpath"
      write(6,100)"1 0 0 setrgbcolor"
      write(6,103)"50 ",Ysize/2+50," moveto"
      write(6,100)"(Unspecified error while reading file ",
     &              bandName,") show"
      call WritePSTail()
      close(1)
      stop
C Goto 201 if the file "bandName" ended before the string "END" was read
201   write(6,100)"/Times-Roman findfont"
      write(6,101)TextSize," scalefont"
      write(6,100)"setfont"
      write(6,100)"newpath"
      write(6,100)"1 0 0 setrgbcolor"
      write(6,103)"50 ",Ysize/2+50," moveto"
      write(6,100)"(Unexpected end of file while reading ",
     &              bandName,") show"
      call WritePSTail()
      stop
      close(1)
C
100   FORMAT(A)
101   FORMAT(F12.6,A)
102   FORMAT(2F12.6,A)
103   FORMAT(A,F12.6,A)
104   FORMAT(A,F5.1,A)
      end
C
      subroutine WritePSBands(qtlName,nkpoints,Xsize,Ysize,
     &                        Efermi,TextSize,Emin,Emax)

C
C
C
      implicit none
      character qtlName*100
      real*8 Xsize,Ysize,Efermi,TextSize,Emin,Emax
      integer nkpoints
C
      integer MaxAtoms,MaxOrbitals
      parameter (MaxAtoms=100,MaxOrbitals=100)
      real RydToEv
      parameter (RydToEv=13.6056981)
C
      real*8 OrbCharacter(MaxAtoms,MaxOrbitals),Thickness,RGBColor(3),
     &       Energy,Eprev,Enow,Enext,ThicknessB,RGBColorB(3),
     &       ptsPerE,ptsPerK
      integer OK,nAtom,nOrb(MaxAtoms),i,j,k,dumi,junki
      character RDWD*(256)
C
      ptsPerE=Ysize/(Emax-Emin)
      ptsPerK=Xsize/(nkpoints-1)
C Open the qtl file
      open(unit=2,file=qtlName,status="old",iostat=OK)
      if(OK.NE.0) then
        write(6,100)"/Times-Roman findfont"
        write(6,101)TextSize,"12 scalefont"
        write(6,100)"setfont"
        write(6,100)"newpath"
        write(6,100)"1 0 0 setrgbcolor"
        write(6,103)"50 ",Ysize/2+50," moveto"
        write(6,100)"(Could not open file",qtlName,") show"
        call WritePSTail()
        stop
      endif
C initiallize some vars to zero
      i=0
      j=0
      do while (i.LE.MaxAtoms)
       i=i+1
       nOrb(i)=0
       do while (j.LE.MaxOrbitals)
        j=j+1
        OrbCharacter(i,j)=0
       enddo
      enddo
C find out how many atoms and orbitals ther are
      nAtom=0
      read(unit=2,fmt=100,iostat=OK,err=200,end=201) RDWD
      do while (index(RDWD,"BAND:").EQ.0)
        if(index(RDWD,"JATOM").NE.0) then
          nAtom=nAtom+1
          nOrb(nAtom)=1
          dumi=1
          do while (index(RDWD(dumi:256),",").NE.0)
            dumi=dumi+index(RDWD(dumi:256),",")
            nOrb(nAtom)=nOrb(nAtom)+1
          enddo
        endif
        read(unit=2,fmt=100,iostat=OK,err=200,end=201) RDWD
      enddo
C add the intersitial character
      nAtom=nAtom+1
      nOrb(nAtom)=1
C read nkpoints k points then read the string BAND: XX if the 
C string BAND: XX can not be read while EOF has been reached stop
      do while(1.EQ.1)
      i=0    
      do while(i.LT.nkpoints)
       i=i+1
       j=0
       do while(j.LT.nAtom)
        j=j+1
        k=0
        read(unit=2,fmt=*,iostat=OK,err=200,end=201) Energy,junki,
     &                            (OrbCharacter(j,k),k=1,nOrb(j))
        Energy=(Energy-Efermi)*RydToEv
      enddo
      call CharacterToThickness(OrbCharacter,Thickness,MaxAtoms)
      call CharacterToRGBColor(OrbCharacter,RGBColor,MaxAtoms)
      if(i.EQ.1)then
        Eprev=Energy
        RGBColorB=RGBColor
        ThicknessB=Thickness
      elseif(i.EQ.2)then
        Enow=Energy
        if((Eprev.GT.Emin).AND.(Eprev.LT.Emax).AND.
     &    (((Eprev+Enow)/2.0).GT.Emin).AND.(((Eprev+Enow)/2.0).LT.Emax))
     &  then
          write(6,105)50+(i-2.0)*ptsPerK,50+(Eprev-Emin)*ptsPerE,
     &                50+(i-1.5)*ptsPerK,50+(0.5*(Enow+Eprev)-Emin)*
     &                ptsPerE,RGBColorB,ThicknessB," L1"
        endif
        RGBColorB=RGBColor
        ThicknessB=Thickness
      else
        Enext=Energy
        if((Enow.GT.Emin).AND.(Enow.LT.Emax).AND.
     &     (0.5*(Enow+Eprev).GT.Emin).AND.
     &     (0.5*(Enow+Eprev).LT.Emax).AND.
     &     (0.5*(Enow+Enext).GT.Emin).AND.
     &     (0.5*(Enow+Enext).LT.Emax))then
          write(6,106)50+(i-2.5)*ptsPerK,50+(0.5*(Eprev+Enow)-Emin)*
     &                ptsPerE,50+(i-2.0)*ptsPerK,50+(Enow-Emin)*
     &                ptsPerE,50+(i-1.5)*ptsPerK,50+(0.5*(Enext+Enow)-
     &                Emin)*ptsPerE,RGBColorB,ThicknessB," L"
        elseif((Enow.GT.Emin).AND.(Enow.LT.Emax).AND.
     &     (0.5*(Enow+Eprev).GT.Emin).AND.
     &     (0.5*(Enow+Eprev).LT.Emax))then
          write(6,105)50+(i-2.5)*ptsPerK,50+(0.5*(Eprev+Enow)-Emin)*
     &                ptsPerE,50+(i-2.0)*ptsPerK,50+(Enow-Emin)*
     &                ptsPerE,RGBColorB,ThicknessB," L1"
        elseif((Enow.GT.Emin).AND.(Enow.LT.Emax).AND.
     &     (0.5*(Enow+Enext).GT.Emin).AND.
     &     (0.5*(Enow+Enext).LT.Emax))then
          write(6,105)50+(i-1.5)*ptsPerK,50+(0.5*(Enext+Enow)-Emin)*
     &                ptsPerE,50+(i-2.0)*ptsPerK,50+(Enow-Emin)*
     &                ptsPerE,RGBColorB,ThicknessB," L1"
        endif
        RGBColorB=RGBColor
        ThicknessB=Thickness
        if(i.EQ.nkpoints)then
         if((Enext.GT.Emin).AND.(Enext.LT.Emax).AND.
     &    (((Enext+Enow)/2.0).GT.Emin).AND.(((Enext+Enow)/2.0).LT.Emax))
     &   then
          write(6,105)50+(i-1.0)*ptsPerK,50+(Enext-Emin)*ptsPerE,
     &                50+(i-1.5)*ptsPerK,50+(0.5*(Enow+Enext)-Emin)*
     &                ptsPerE,RGBColorB,ThicknessB," L1"
         endif
        endif
        Eprev=Enow
        Enow=Enext
      endif
      enddo
C read the string BAND: XX or iff EOF finished
      read(unit=2,fmt=100,iostat=OK,err=200,end=300) RDWD
      enddo
300   continue
      close(2)
      return
C Goto 200 if the file "qtlName" had an error upon reading
200   write(6,100)"/Times-Roman findfont"
      write(6,101)TextSize," scalefont"
      write(6,100)"setfont"
      write(6,100)"newpath"
      write(6,100)"1 0 0 setrgbcolor"
      write(6,103)"50 ",Ysize/2+50," moveto"
      write(6,100)"(Unspecified error while reading file ",
     &              qtlName,") show"
      call WritePSTail()
      close(2)
      stop
C Goto 201 if the file "qtlName" ended before the end was expected
201   write(6,100)"/Times-Roman findfont"
      write(6,101)TextSize," scalefont"
      write(6,100)"setfont"
      write(6,100)"newpath"
      write(6,100)"1 0 0 setrgbcolor"
      write(6,103)"50 ",Ysize/2+50," moveto"
      write(6,100)"(Unexpected end of file while reading ",
     &              qtlName,") show"
      call WritePSTail()
      stop
      close(2)
C
100   FORMAT(A)
101   FORMAT(F12.6,A)
102   FORMAT(2F12.6,A)
103   FORMAT(A,F12.6,A)
104   FORMAT(A,F5.1,A)
105   FORMAT(8F10.4,A)
106   FORMAT(10F10.4 A)
      end
CC
CC
CC
CC
CC
CC
CC
CC
CC
CC
C      subroutine ReadDefinitions(qtlName,bandName,Xsize,Ysize,Emin,Emax,
C     &                           Efermi,MajorTicks,MinorTicks,TextSize)
C      implicit none
C      character qtlName*100,bandName*100
C      integer MinorTicks
C      real*8 Xsize,Ysize,Emin,Emax,Efermi,MajorTicks,TextSize
C      qtlName="Sr.qtl"
C      bandName="Sr.klist_band"
C      Xsize=400
C      Ysize=300
C      Emin=-7.0
C      Emax=3.0
C      Efermi=0.70187
C      MajorTicks=2.0
C      MinorTicks=3
C      TextSize=12
C      end
CC
C      subroutine CharacterToThickness(OrbCharacter,Thickness,natoms)
C      implicit none
C      integer natoms
C      real*8 OrbCharacter(natoms,*),Thickness
CC
C      Thickness=0.2
CC
C      end
CC
C      subroutine CharacterToRGBColor(OrbCharacter,RGBColor,natoms)
C      implicit none
C      integer natoms
C      real*8 OrbCharacter(natoms,*),Red,Green,Blue,RGBColor(3)
CC
C       Red=OrbCharacter(1,1)
C       Green=OrbCharacter(2,1)
C       Blue=OrbCharacter(3,1)+OrbCharacter(4,1)
CC
C      RGBColor(1)=Red
C      RGBColor(2)=Green
C      RGBColor(3)=Blue
C      end


